
{+--------------------------------------------------------------------------+
  Unit de dmonstration Edition d'image (luminosit contraste et filtres
  Jean Yves Quinec 28/10/2000 - j.y.q@wanadoo.fr
 +--------------------------------------------------------------------------+

  Le premier principe permettant de simplifier la gestion des bitmaps consiste
   les transformer systmatiquement en 16 millions de couleurs, c'est  dire
  en  pixels de 3 Bytes (R,G,B red green blue) soit 24 bits,  l'aide de

    Tbitmap.pixelformat = pf24bit

 Technique utilise pour manipuler les pixels

 Bitmap + Scanline

   A partir de Delphi3 nous pouvons utiliser Scanline[ligne] qui fournit
   un pointeur (sans type) vers un tableau des lignes du bitmap.
   Noter que scanline fonctionne dans le "bon" sens (de haut en bas) alors
   que le bitmap stocke ses lignes de bas en haut.

   - A nous de savoir le nombre d'octets (ou de bits) qui correspondent 
     un pixel sur une ligne. D'o la simplification  3 bytes pour n'avoir
      traiter que ce cas.

   - L'API Windows fournit la structure TRGBtriple (3 bytes), qui dcrit
     les trois couleurs :  .Rgbtred, .Rgbtgreen, .Rgbtblue dans un bitmap.
     Nous dclarons donc le type :

      TRGBArray = ARRAY[0..0] OF TRGBTriple;
      pRGBArray = ^TRGBArray;     // type pointeur vers tableau 3 octets

   -  Accs  un pixel
      Pour spcifier le rang de la colonne x, scanlines utilise un tableau
      d'integer. Pour obtenir un pixel isol en position (x, y) on a :
         var
           apixel : TRGBTRiple;
         ...........
         begin
           if (x >= 0) AND (y >= 0) AND (x < bitmap.width-1) AND
              (y < Bitmap.height-1) then     // vrif limites
              apixel := pRGBArray(Bitmap.scanline[y])[x];
         end;

      Explications :

        Le rsultat fait 3 bytes puisque nous travaillons en 16 millions de
        couleurs, donc apixel est de type TRGBTriple.
        Scanline est un pointeur sans type vers un tableau de lignes.

        La syntaxe pRGBArray(Bitmap.scanline[y]) effectue un transtypage,
        c'est  dire que nous prcisons le type du pointeur de tableau de
        lignes. Cette information est indispensable au compilateur pour
        qu'il effectue ses calculs d'adresse et de longueur.

        Maintenant que nous avons la ligne du tableau, [x] indique la
        colonne voulue.
        Noter les limites Bitmap.width-1 et Bitmap.height-1.

    -  Travail ligne par ligne avec manipulation des couleurs
       On veut claircir le Bitmap ==> +8 sur chaque couleur
         var
           row : pRGBarray;     // pointeur sur une ligne
           R,G,B : integer;     // couleurs
         ........
         For y := 0 to bitmap.height-1 do     // boucle lignes
         begin
           row := Bitmap.scanline[y];  // pointeur vers la ligne
           for x := 0 to bitmap.width-1 do   // boucle colonnes
           begin
             R := row[x].RGBtRed  +8;  // row[x] = pixel de type TRGbtriple
             G := row[x].RGBtGreen+8;
             B := row[x].RGBtBlue +8;
             // contrle limite des couleurs. Saturation teste en premier
            if R > 255 then R := 255 else if R < 0 then R := 0;
            if G > 255 then G := 255 else if G < 0 then G := 0;
            if B > 255 then B := 255 else if B < 0 then B := 0;
            row[x].RGBtRed   := R;
            row[x].RGBtGreen := G;
            row[x].RGBtBlue  := B;
          end;
         end;

    - Optimisation lorsque la manipulation du bitmap ne s'effectue pas
      ligne par ligne.

       Dans ce cas, le calcul du scanline  chaque pixel (voir accs
        un pixel ci-dessus), consomme du processeur. Pour aller 2 ou 3
       fois plus vite, comme avec les tableaux Tscana et Tscanb de ce
       programme, on peut calculer d'avance les pointeurs scanline et
       les stocker dans un tableau  nous.
       Note : selon le mme principe, on peut calculer des tableaux de sinus
              et de cosinus en dbut de programme.

       var
        // attention, pas de bitmap plus haut que 2048 lignes
        Var   // globale
          Lignes : array[0..2047] of pRGBArray;
        .......
        // prcalcul des scanline
          For i := 0 to Bitmap.heigh-1 do Lignes[y] := bitmap.scanline[y];
        ......
        //  Accs rapide  un pixel
        Function Tform1.getpixel(ax,ay : integer): TRGBTRiple;
        const
           Filler : array[0..2] of byte = (0,0,0);   // noir par dfaut
        begin
          if (ax >= 0) AND (ay >= 0) AND             // limites
            (ax < bitmap.width-1)  AND (ay < Bitmap.height-1) then
          result := Lignes[ay,ax]
          else
          result := Trgbtriple(Filler);  // transtypage des 3 bytes
        end;

--}

unit UEstamp1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ExtDlgs, jpeg, Grids;
type
  TForm1 = class(TForm)
    Panel1: TPanel;
    BtQuitter: TButton;
    BtLecture: TButton;
    Panel2: TPanel;
    PaintBox1: TPaintBox;
    Panel3: TPanel;
    Btluminosite: TButton;
    UpDownR: TUpDown;
    UpDownG: TUpDown;
    UpDownB: TUpDown;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Chkensemble: TCheckBox;
    Btcontraste: TButton;
    Btraz: TButton;
    Btreset: TButton;
    Btmelange: TButton;
    Btenregistrer: TButton;
    UpDownMel: TUpDown;
    Edit6: TEdit;
    Btmixlimage: TButton;
    RadioGroup1: TRadioGroup;
    Label2: TLabel;
    Btappliquer: TButton;
    Chkgris: TCheckBox;
    ChkNeg: TCheckBox;
    RadioGroup2: TRadioGroup;
    Button1: TButton;
    ColorDialog1: TColorDialog;
    Btcolorise: TButton;
    Panel4: TPanel;
    Edit8: TEdit;
    UpDownC: TUpDown;
    PD: TPanel;
    PF00: TPanel;
    PF01: TPanel;
    PF02: TPanel;
    PF03: TPanel;
    PF04: TPanel;
    PF05: TPanel;
    PF06: TPanel;
    PF07: TPanel;
    PF08: TPanel;
    Btbruit: TButton;
    Edit7: TEdit;
    UpDownBruit: TUpDown;
    Btgauss: TButton;
    BtCouleurs: TButton;
    Btgamma: TButton;
    Btnouveau: TButton;
    Btmixdegrad: TButton;
    Btsauver: TButton;
    Btbrume: TButton;
    PF09: TPanel;
    PF10: TPanel;
    PF11: TPanel;
    PF12: TPanel;
    PF13: TPanel;
    PF14: TPanel;
    PF15: TPanel;
    PF16: TPanel;
    PF17: TPanel;
    PF18: TPanel;
    PF19: TPanel;
    PF20: TPanel;
    PF21: TPanel;
    PF22: TPanel;
    PF23: TPanel;
    PF24: TPanel;
    Btundo: TButton;
    SaveDialog1: TSaveDialog;
    Btparamjpg: TButton;
    Btcalcul: TButton;
    BtsymO: TButton;
    Btsymx: TButton;
    BtsymY: TButton;
    PanelA: TPanel;
    Label1: TLabel;
    panel5: TPanel;
    Shap2: TShape;
    Shap3: TShape;
    Shap4: TShape;
    Shap5: TShape;
    Shap6: TShape;
    Shap7: TShape;
    Shap0: TShape;
    Shap1: TShape;
    Bevel2: TBevel;
    Bevel1: TBevel;
    Shape1: TShape;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure BtQuitterClick(Sender: TObject);
    procedure BtLectureClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure BtredoClick(Sender: TObject);
    procedure BtresetClick(Sender: TObject);
    procedure BtundoClick(Sender: TObject);
    procedure BtluminositeClick(Sender: TObject);
    procedure UpDownRClick(Sender: TObject; Button: TUDBtnType);
    procedure UpDownGClick(Sender: TObject; Button: TUDBtnType);
    procedure UpDownBClick(Sender: TObject; Button: TUDBtnType);
    procedure ChkensembleClick(Sender: TObject);
    procedure BtcontrasteClick(Sender: TObject);
    procedure BtrazClick(Sender: TObject);
    procedure BtmelangeClick(Sender: TObject);
    procedure BtenregistrerClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure BtbrumeClick(Sender: TObject);
    procedure BtmixlimageClick(Sender: TObject);
    procedure RadioGroup2Click(Sender: TObject);
    procedure BtappliquerClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Panel4Click(Sender: TObject);
    procedure BtcoloriseClick(Sender: TObject);
    procedure BtbruitClick(Sender: TObject);
    procedure PDMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PF00MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtgaussClick(Sender: TObject);
    procedure BtCouleursClick(Sender: TObject);
    procedure BtgammaClick(Sender: TObject);
    procedure BtnouveauClick(Sender: TObject);
    procedure BtmixdegradClick(Sender: TObject);
    procedure BtsauverClick(Sender: TObject);
    procedure BtparamjpgClick(Sender: TObject);
    procedure BtcalculClick(Sender: TObject);
    procedure BtsymOClick(Sender: TObject);
    procedure BtsymxClick(Sender: TObject);
    procedure BtsymYClick(Sender: TObject);
    procedure Shap0MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure panel5MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    procedure WMEraseBkgnd(var m: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure Dimensionpaintbox;
    procedure Affichebmp2;
    procedure Initbitmaps(Action: integer);
    procedure effets;
    procedure Couleurs(PctR, PctG, PctB : integer; afonction : integer);
    procedure origine1sur2;
    Procedure choixfiltre(numero : integer);
    Procedure affichefiltre;
    Procedure estampage;
    Procedure Mixage;
    procedure vitrage;
  public
    procedure melangeBmp(bmpa, bmpb : tbitmap; rectb: trect);
  end;

const
 CRPLUS = 1;
 CRPIPETTE = 2;

var
  Form1: TForm1;
  bmp0 : Tbitmap;   // bitmap lu Par ulecture
  Rbmp0 : Trect;    // utilis par Form4

  firsttime : boolean;
  refreshdir : boolean;

implementation

{$R *.DFM}
{$R Curseurs.res}  // curseur
uses Ulecture, Uestamp4, UGauss, Uestamp2, Uestamp5, Uestamp6;

type
  TRGBArray = ARRAY[0..0] OF TRGBTriple;   // lment de bitmap (API windows)
  pRGBArray = ^TRGBArray;     // type pointeur vers tableau 3 octets 24 bits

Var
  bmp1 : Tbitmap;   // mouveau bitmap lu
  bmp2 : Tbitmap;   // bitmap  afficher dans la paintbox
  bmp3 : Tbitmap;   // bitmap de manoeuvre pour effets spciaux
  afficher : boolean;
  // tableaux de scanlines 3 octets pour optimisation
  // les images n('auront pas plus de 2048 en hauteur
  Tscana : array[0..2047] of pRGBArray;
  Tscanb : array[0..2047] of PRGBArray;
  ctrf   : integer;
  moving : boolean;
  melange : boolean;
  estamper : boolean;
  startx, starty : integer;
  posx, posy, posw, posh : integer;
  // Valeurs des Panels de saisie numrique
  Dv : integer;                   // diviseur coefficients matrice
  PF : array[0..24] of integer;    // coefficients matrice filtre maxi 5x5
  gamma : array[0..255] of integer;  // correction gamma
  // Panneau 8 directions
  Direction : integer;
  Decalage  : integer;

procedure TForm1.FormCreate(Sender: TObject);
var
  i : integer;
  ri : single;
  s : string;
  c : Tcomponent;
begin
  screen.cursors[crplus] := loadcursor(Hinstance, 'PLUS');
  screen.cursors[crpipette] := loadcursor(Hinstance, 'PIPETTE');
  paintbox1.cursor := crpipette;
  For i := 0 to 24 do
  begin
    s := inttostr(i);       // Fabrique le nom des composants PF0 .. PF24
    if length(s) < 2 then s := '0'+s;
    s := 'PF'+s;
    c := findcomponent(s);
    if c <> Nil then  (c as Tpanel).cursor := crplus;
  end;
  PD.cursor := crplus;
  Dv := 1;
  For i := 0 to 8 do PF[i] := 0;  // filtre neutre
  PF[4] := 1;
  moving := false;
  melange := false;
  afficher := true;
  refreshdir := false;
  estamper := false;
  randomize;
 { Pour visualiser la courbe gamma, placer un Timage1 de dimension 256x256
   et enlever les commentaires des 4 lignes concernant image1 }
 // Image1.canvas.Moveto(255,255);
 // Image1.canvas.lineto(0,0);
 // Image1.canvas.pen.color := clred;
  For i := 0 to 255 do  // prcalcul gamma
  begin
    ri := i;
    gamma[i] := round(power(i/255,0.5)*255);
   // Image1.canvas.lineto(i, gamma[i]);
  end;
  ctrf := 0;
  firsttime := true;
  direction := 7;
  decalage  := 2;
  bmp0 := tbitmap.create; bmp0.width  := 8;  Bmp0.height := 8;
  bmp1 := tbitmap.create; bmp1.width  := 8;  Bmp1.height := 8;
  bmp2 := Tbitmap.create; bmp2.width  := 8;  Bmp2.height := 8;
  bmp3 := tbitmap.create; bmp3.width  := 8;  Bmp3.height := 8;
  chkgris.checked := false;
  chkneg.checked  := false;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  btlectureclick(sender);
  firsttime := false;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  bmp0.free;
  bmp1.free;
  bmp2.free;
  bmp3.free;
end;

procedure TForm1.BtQuitterClick(Sender: TObject);
begin
  close;
end;

procedure TForm1.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
  if firsttime then exit;
  affichebmp2;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  dimensionpaintbox;
end;

procedure Tform1.DimensionPaintbox;
begin
  case radiogroup1.itemindex of
  0 : begin
      Paintbox1.width  := bmp2.width div 2;
      Paintbox1.height := bmp2.height div 2;
      end;
  1 : begin
      Paintbox1.width  := bmp2.width;
      Paintbox1.height := bmp2.height;
      end;
  2 : begin
      Paintbox1.width  := bmp2.width * 2;
      Paintbox1.height := bmp2.height* 2;
      end;
  end;
  // centrage de la paintbox
  Paintbox1.left := (panel2.width - paintbox1.width) div 2;
  Paintbox1.top  := (panel2.height - paintbox1.height) div 2;
  paintbox1.invalidate;  // affiche le rsultat
end;

procedure Tform1.afficheBmp2;
begin
  if not afficher then exit;
  with paintbox1.canvas do
  begin
    copymode := srccopy;
    If radiogroup1.itemindex = 1 then
    draw(0,0,bmp2)
    else
     stretchdraw(rect(0,0,paintbox1.width, paintbox1.height),bmp2);
  end;
end;

// zoom
procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  dimensionpaintbox;
end;

//---------------------  Gestion des boutons ----------------------
procedure TForm1.BtLectureClick(Sender: TObject);
begin
  IF form3.showmodal = mrok then
  begin
    Initbitmaps(0);
    with form3.filelistbox1 do
    begin
     if itemindex > -1 then Form1.caption := 'Estampe - '+items[itemindex];
   end;
  end;
end;


procedure TForm1.BtparamjpgClick(Sender: TObject);
begin
  form2.showmodal;
end;

// Enregistrement de l'image modifie au format .jpg - compression 10%
procedure TForm1.BtenregistrerClick(Sender: TObject);
var
  JP : TJPEGImage;
begin
  { enregistrer bmp2 au format .JPEG  }
 saveDialog1.filename := form3.filelistbox1.items[form3.filelistbox1.itemindex];
  if savedialog1.execute then
  begin
    try
      JP := TJPEGImage.Create;             // Cre un TJPEGImage temporaire
      JP.CompressionQuality := Form2.UpdownJpg.position;  // compression
      if form2.radiogroup1.itemindex = 0 then
        JP.ProgressiveEncoding := false
      else
        JP.ProgressiveEncoding := true;
      JP.Assign(bmp2);                    // Assigne le bitmap au JPEG
      JP.SaveToFile(saveDialog1.FileName);  // enregistrement
    finally
      JP.Free;                            // Free  JPEG
    end;
    refreshdir := true;
  end;
end;

// Prserve bmp1 et utilise bmp0 pour mlanger avec bmp2
// cette image sera dplaable sur bmp2 sauf si pavage
procedure TForm1.BtmixdegradClick(Sender: TObject);
begin
  Form5.caption := 'Nouvelle image  mixer dans '+inttostr(bmp2.width)
            +' x '+inttostr(bmp2.height);
  IF form5.showmodal = mrok  then
  begin
    bmp0.width  := form5.updown1.position;
    bmp0.height := form5.updown2.position;
    bmp0.pixelformat := pf24bit;
    form5.degrader(bmp0.canvas, rect(0,0, bmp0.width, bmp0.height));
    mixage;
  end;
end;

procedure TForm1.BtmixlimageClick(Sender: TObject);
begin
 IF form3.showmodal = mrok then mixage;
end;

procedure Tform1.mixage;
var
  Rfit  : trect;
  kzoom : single;
begin
  fitrect(Rbmp0, kzoom, Form4.panel1.width, form4.panel1.height,
               bmp0.width, bmp0.height,8,8, false);
  Form4.paintbox1.left   := Rbmp0.left;
  Form4.paintbox1.top    := Rbmp0.top;
  Form4.paintbox1.width  := Rbmp0.right - Rbmp0.left;
  Form4.paintbox1.height := Rbmp0.bottom - Rbmp0.top;
  Rbmp0 := bounds(0,0,Form4.paintbox1.width, form4.paintbox1.height);
  Form4.label10.caption := '('+inttostr(bmp0.width)+'x'+inttostr(bmp0.height)
  +') dans ('+inttostr(bmp2.width)+'x'+inttostr(bmp2.height)+')';
  IF form4.showmodal = mrok then
  begin
    bmp3.canvas.draw(0,0,bmp2);   // sauve
    // Miroirs ventuels sur bmp0
    IF form4.radiomiroir.itemindex = 1 then  // Horiontal (axe vertical)
    Bmp0.Canvas.StretchDraw(Rect(bmp0.width-1, 0, -1, Bmp0.height), bmp0);
    IF form4.radiomiroir.itemindex = 2 then  // vertical (axe horizontal)
    Bmp0.Canvas.StretchDraw(Rect(0, Bmp0.height-1,Bmp0.width,-1), bmp0);
    // mlanger l'image centre dans le rectangle dplaable par la suite
    // Rbmp0, aprs avoir appliqu le facteur de zoom de Form4 ou
    // le choix ajuster en hauteur ou largeur
    case  Form4.radioDimension.itemindex of
    0 : begin         // zoom
        posw := (bmp0.width*Form4.updownzoom.position) div 100;
        posh := (bmp0.height*Form4.updownzoom.position) div 100;
        posx := (bmp2.width-posw) div 2; // centr au dpart
        posy := (bmp2.height-posh) div 2;
        end;
    1 : begin        // stretch
        posx := 0;
        posy := 0;
        posw := bmp2.width;
        posh := bmp2.height;
        end;
    2 : begin        // choix meilleure taille et centrage
        FitRect(rfit, kzoom, bmp2.width, bmp2.height, bmp0.width, bmp0.height,
                0,0,false);
        posx := rfit.left;
        posy := rfit.top;
        posw := rfit.right  - rfit.left;
        posh := rfit.bottom - rfit.top;
        end;
    end;
    Rbmp0 := bounds(posx, posy, posw, posh);
    melangebmp(bmp2, bmp0, Rbmp0);
    affichebmp2;
  end
end;

procedure TForm1.BtredoClick(Sender: TObject);
begin
  effets;
end;

procedure TForm1.BtresetClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  Initbitmaps(1);
end;

procedure TForm1.BtundoClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  Bmp2.canvas.draw(0,0,Bmp3);
  affichebmp2;
end;

procedure TForm1.BtluminositeClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crdefault;
  couleurs(updownR.position, updownG.position, updownB.position, 0);
end;

procedure TForm1.BtgammaClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 1);
end;

procedure TForm1.BtcontrasteClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 2);
end;

procedure TForm1.BtcoloriseClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 3);
end;

procedure TForm1.BtbruitClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 4);
end;

procedure TForm1.BtbrumeClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 5);
end;

procedure TForm1.BtCouleursClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  couleurs(updownR.position, updownG.position, updownB.position, 6);
end;

procedure TForm1.vitrage;
var
  i : integer;
  bmp4 : Tbitmap;
begin
 try
  bmp4 := tbitmap.create;       // for undo
  bmp4.width  := bmp2.width;
  bmp4.height := bmp2.height;
  bmp4.pixelformat := pf24bit;
  bmp4.canvas.draw(0,0,bmp2);
  melange := false; paintbox1.cursor := crpipette;
  afficher := false;
  updownR.position := 10;
  updownG.position := 10;
  updownB.position := 10;
  for i := 1 to 4 do
  begin
    effets;
    if i = 4 then afficher := true;
    Origine1sur2;
  end;
   btgammaclick(form1);
   bmp3.canvas.draw(0,0,bmp4);  // for undo
 finally
   bmp4.free;
 end;
end;

//--------------------- section fonctions  --------------------------

// action = 0 tout initialiser avec Bmp0
// action = 1 initialiser Bmp3 et conserver bmp2
Procedure Tform1.initbitmaps(Action : integer);
var
 y : integer;
begin
  If action = 0 then
  begin
    bmp1.width := bmp0.width;
    bmp1.height := bmp0.height;
    bmp1.pixelformat := pf24bit;
    bmp1.canvas.draw(0,0,bmp0);
  end;
  // bmp2 est le bitmap rsultat
  bmp2.width  := bmp1.width;
  bmp2.height := bmp1.height;
  bmp2.pixelformat := pf24bit;
  bmp2.canvas.draw(0,0,bmp1);
  DimensionPaintbox;
  Affichebmp2;
  bmp3.width := bmp2.width;
  bmp3.height := bmp2.height;
  Bmp3.pixelformat := pf24bit;
  IF action = 0 then bmp3.canvas.draw(0,0,bmp2);
  // optimisation travail sur plusieurs lignes: prcalcul pointeurs de scanline
  For y := 0 to bmp2.height-1 do
  begin
    Tscana[y] := bmp3.scanline[y];   // origine
    Tscanb[y] := bmp2.scanline[y];   // destination
  end;
end;

// Mlange :  Bmpa est le rsultat et ne change pas de taille
// On mlange Bmpb avec bmpa de 2 manires :  tiles ou copy en appliquant le
// Pourcentage de mlange dfini dans form4.
// Si copy - Bmpb subit un stretch pour tenir dans le rectangle rectb
// Si masque on ne copie pas les pixels de la couleur form4.pancolmask.color
// avec tolrance de updown tolere
procedure Tform1.melangeBmp(bmpa, bmpb: tbitmap; rectb : Trect);
var
  bmp4 : Tbitmap;
  w, h : integer;
  x, y : integer;
  x0 : integer;
  Rowa : Prgbarray;
  Rowb : Prgbarray;
  r, g, b : integer;
  Rt, Gt, Bt : integer;
  d : integer;
  pct : integer;
  R1 : Trect;    // clip rectangle
begin
 try
  bmp4 := tbitmap.create;
  bmp4.width  := bmpa.width;
  bmp4.height := bmpa.height;
  bmp4.pixelformat := pf24bit;
  case form4.radiostyle.itemindex of
   0,1 : begin        // copie dans bmp4
        R1 := rectb;
        melange := true;
        paintbox1.cursor := crhandpoint;
        bmp4.canvas.stretchdraw(R1, bmpb);
      end;

   2 : begin         //  pavage avec offset
        R1 := rect(0,0,bmp4.width, bmp4.height);  // mlanger toute l'image
        w := bmpb.width;
        h := bmpb.height;
        y := Form4.updownOffsety.position;
        if y > 0 then y := y-h;
        x0 := Form4.updownOffsetx.position;
        if x0 > 0 then X0 := x0-w;
        while y < bmp4.height do
        begin
          x := x0;
          while x < bmp4.width do
          begin
            Bmp4.canvas.draw(x,y, bmpb);
            inc(x, w);
          end;
          inc(y, h);
        end;
      end;
  end;  // case
  IF r1.left < 0 then r1.left := 0;
  if r1.top  < 0 then r1.top := 0;
  if r1.right  > bmp4.width then r1.right := bmp4.width;
  if r1.bottom > bmp4.height then r1.bottom := bmp4.height;
  pct :=  Form4.updownMix.position;
  IF form4.radiostyle.itemindex = 1 then  // avec masque
  begin
    d  := Form4.updowntolere.position;
    Rt := GetRValue( ColorToRGB(form4.pancolmask.color));
    Gt := GetGValue( ColorToRGB(form4.pancolmask.color));
    Bt := GetBValue( ColorToRGB(form4.pancolmask.color));
    For y := R1.top to R1.bottom-1 do
    begin
      rowb := Bmp4.scanline[y];
      rowa := Bmpa.scanline[y];
      for x := R1.left to R1.Right-1 do
      begin
        R := rowb[x].RgbtRed;
        G := rowb[x].Rgbtgreen;
        B := rowb[x].Rgbtblue;
        if (abs(R-Rt) <= d) and (abs(G-Gt) <= d) and (abs(B-Bt) <= d) then
        begin   // masque : prendre le pixel d'origine
          R := rowa[x].Rgbtred*100;
          G := rowa[x].Rgbtgreen*100;
          B := rowa[x].Rgbtblue*100;
        end
        else
        begin    // mlanger avec poucentage
          R := rowa[x].Rgbtred*(100-pct)   +rowb[x].RgbtRed   *pct;
          G := rowa[x].Rgbtgreen*(100-pct) +rowb[x].Rgbtgreen *pct;
          B := rowa[x].Rgbtblue*(100-pct)  +rowb[x].Rgbtblue  *pct;
        end;
        rowa[x].Rgbtred   := R div 100;
        rowa[x].Rgbtgreen := G div 100;
        rowa[x].Rgbtblue  := B div 100;
      end;
    end;
  end
  else
  For y := R1.top to R1.bottom-1 do
  begin
    rowb := Bmp4.scanline[y];
    rowa := Bmpa.scanline[y];
    for x := R1.left to R1.Right-1 do
    begin
      R := rowa[x].Rgbtred*(100-pct)   +rowb[x].RgbtRed   *pct;
      G := rowa[x].Rgbtgreen*(100-pct) +rowb[x].Rgbtgreen *pct;
      B := rowa[x].Rgbtblue*(100-pct)  +rowb[x].Rgbtblue  *pct;
      rowa[x].Rgbtred   := R div 100;
      rowa[x].Rgbtgreen := G div 100;
      rowa[x].Rgbtblue  := B div 100;
    end;
  end;
 finally
   bmp4.free;
 end;
end;

// met 1 pixel sur 2 de Bmp1 dans bmp2
procedure Tform1.origine1sur2;
var
  x, y : integer;
  Rowa : Prgbarray;
  Rowb : Prgbarray;
begin
  For y := 0 to bmp2.height-1 do
  begin
    rowa := Bmp1.scanline[y];
    rowb := Bmp2.scanline[y];
    for x := 0 to bmp2.width-1 do
    begin
      if (x+y) mod 2 = 0 then
      begin
        rowb[x].Rgbtred   := rowa[x].Rgbtred;
        rowb[x].Rgbtgreen := rowa[x].Rgbtgreen;
        rowb[x].Rgbtblue  := rowa[x].Rgbtblue;
      end;
    end;
  end;
  affichebmp2;
end;


// procedure Effets
// Traite un pixel selon coefficients de la matrice de filtrage
// de dimensions utiles 3x3 ou 5x5 et d'un coefficient diviseur.
// en principe  somme-des-coefficients-du-filtre / diviseur = 1
// Le pixel trait est au centre de la matrice.
// Bmp3 origine et Bmp2 rsultat
procedure Tform1.Effets;
const
  M3 : array[0..24] of integer = (1,1,1,1,1,   // pour savoir si matrice 3x3
                                  1,0,0,0,1,
                                  1,0,0,0,1,
                                  1,0,0,0,1,
                                  1,1,1,1,1);
var
  x, y : integer;
  R, G, B : integer;
  RR, GG , BB : integer;
  i : integer;           // indice pour la matrice coefficients
  ix, iy, dx, dy, d : integer;      // positions matrice glissante
begin
  if dv = 0 then dv := 1;   // le diviseur ne doit pas tre 0
  if estamper then
  begin
    estampage;
    exit;
  end;
  screen.cursor := crHourglass;
  // rduction de la matrice  la taille 3x3 pour optimisation
  dx := 0;
  For i := 0 to 24 do if (PF[i] AND M3[i]) <> 0 then inc(dx);
  if dx = 0 then d := 1 else d := 2 ;  // dcalage autour du centre
  bmp3.canvas.draw(0,0,bmp2);      // on sauve bmp2 pour dfaire
  For y := 0 to bmp3.height-1 do   // -1 pixel pour dimension bitmap y
  for x := 0 to bmp3.width-1 do    // -1 pixel pour dimension bitmap x
  begin
    RR := 0; GG := 0; BB := 0;
    for dy := -d to d do
    for dx := -d to d do
    begin
      // position pixel trait
      iy := y+dy;
      ix := x+dx;
      // rcupration et addition couleurs spares des 9, ou 25 pixels
      if (iy >= 1) and (iy <= Bmp3.height-1) and  // vrif limites
         (ix >= 1) and (ix <= Bmp3.width-1) then
      begin
        R := Tscana[iy, ix].RgbtRed;
        G := Tscana[iy, ix].Rgbtgreen;
        B := Tscana[iy, ix].Rgbtblue;
      end
      else
      begin
        R := Tscana[y, x].RgbtRed;  // si en dehors on prend le pixel trait
        G := Tscana[y, x].Rgbtgreen;
        B := Tscana[y, x].Rgbtblue;
      end;
      i := 12+dy*5+dx;       // position coefficient dans la matrice du filtre
      RR := RR + R * PF[i];   // multiplier par le coefficient matrice
      GG := GG + G * PF[i];
      BB := BB + B * PF[i];
    end;                // fin traitement pixels
    RR := RR div Dv;
    GG := GG div Dv;
    BB := BB div DV;
    if chkgris.checked then    // mise en niveaux de gris
    begin
      RR := (RR+GG+BB) div 3;
      GG := RR;
      BB := RR;
    end;
    if RR > 255 then RR := 255 else if RR < 0 then RR := 0;
    if GG > 255 then GG := 255 else if GG < 0 then GG := 0;
    if BB > 255 then BB := 255 else if BB < 0 then BB := 0;
    IF chkneg.checked then     // mise en ngatif
    begin
      RR := 255-RR;
      GG := 255-GG;
      BB := 255-BB;
    end ;
    Tscanb[y,x].rgbtred   := RR;
    Tscanb[y,x].rgbtgreen := GG;
    Tscanb[y,x].rgbtblue  := BB;
  end;
  screen.cursor := crDefault;
  affichebmp2;
end;

// Ici on traite un pixel  la fois donc les lignes sont rowa et rowb au lieu
// tableaux de scanlines
procedure Tform1.Couleurs(PctR, PctG, PctB : integer; afonction: integer);
var
  x, y : integer;
  Rowa : Prgbarray;
  Rowb : Prgbarray;
  R,G,B : integer;
  R0, G0, B0 : byte;
  Cmax, cmin : integer;
  somme0, k0 : integer;
  bruit, alea: integer;
  somme: integer;
  kR, kG, kB : integer;  // pourcentages en units de 256
  //--------------------------------------
  function mini(a,b : integer): integer;
  begin
    if a < b then result := a else result := b;
  end;
  function maxi(a,b : integer): integer;
  begin
    if a > b then result := a else result := b;
  end;
  //--------------------------------------
begin
  // couleur de colorisation
  R0 := GetRValue( ColorToRGB(panel4.color));
  G0 := GetGValue( ColorToRGB(panel4.color));
  B0 := GetBValue( ColorToRGB(panel4.color));
  somme0 := R0+G0+B0;             // total couleurs de colorisation
  k0 := updownc.position;         // % couleur
  bruit := updownbruit.position*2;  // % de bruit en couleur
  alea := random(97) mod 2;
  bmp3.canvas.draw(0,0,bmp2);     // on reprend Bmp2
  KR := (Pctr*256) div 100;       // pourcentage en valeurs 256
  KG := (Pctg*256) div 100;
  KB := (Pctb*256) div 100;
  For y := 0 to bmp3.height-1 do
  begin
    rowa := Bmp3.scanline[y];
    rowb := Bmp2.scanline[y];
    for x := 0 to bmp3.width-1 do
    begin
      R := rowa[x].RgbtRed;
      G := rowa[x].Rgbtgreen;
      B := rowa[x].Rgbtblue;
      // effectue les calculs
      case afonction of
      0 : begin     // luminosit
          somme := (R+G+B+2) div 3;
          R := R+(somme * kR) div 256;
          G := G+(somme * kG) div 256;
          B := B+(somme * kB) div 256;
          end;
      1 : begin     // Correction gamma
          somme := gamma[(R+G+B) div 3];
          R := R+(somme * kR) div 256;
          G := G+(somme * kG) div 256;
          B := B+(somme * kB) div 256;
          end;
      2 : begin  // contraste
          R := R+((R-127)*kR) div 256;  // % appliqu  cart moyenne de 127
          G := G+((G-127)*kG) div 256;
          B := B+((B-127)*kB) div 256;
          end;
      3 : Begin  // colorisation updownc % avec la couleur de Panel4
          somme := R+G+B;  // luminosit
          R := (R*(100-k0) +(somme*R0*k0) div somme0) div 100;
          G := (G*(100-k0) +(somme*G0*k0) div somme0) div 100;
          B := (B*(100-k0) +(somme*B0*k0) div somme0) div 100;
          end;
      4 : begin  // bruit alatoire uniforme
            R := R+ random(bruit) - bruit div 2;
            G := G+ random(bruit) - bruit div 2;
            B := B+ random(bruit) - bruit div 2;
          end;
      5 : begin
            R := (255 + R*3) div 4;
            G := (255 + G*3) div 4;
            B := (255 + B*3) div 4;
          end;
      6 : begin    // saturation couleurs
          cmax := maxi(R, maxi(G,B));
          cmin := mini(R, mini(G,B));
          somme := (cmax + cmin) div 2;
          R := R+((R-somme)*kR*2) div 256;
          G := G+((G-somme)*kG*2) div 256;
          B := B+((B-somme)*kB*2) div 256;
          end;
      end;
      // Validit des couleurs - Le plus frquent est la saturation
      if R > 255 then R := 255 else if R < 0 then r := 0;
      if G > 255 then G := 255 else if G < 0 then g := 0;
      if B > 255 then B := 255 else if B < 0 then b := 0;
      rowb[x].Rgbtred   := R;
      rowb[x].Rgbtgreen := G;
      rowb[x].Rgbtblue  := B;
    end;
  end;
  affichebmp2;
end;

Procedure Tform1.estampage;
var
  R,G,B,R2,G2,B2 : integer;
  R0,G0,B0 : integer;
  x, y, x2, y2 : integer;
  decalx , decaly : integer;
begin
  case direction of
  0 : begin decalx :=  decalage; decaly :=  0       ; end;
  1 : begin decalx :=  decalage; decaly := -decalage; end;
  2 : begin decalx :=  0       ; decaly := -decalage; end;
  3 : begin decalx := -decalage; decaly := -decalage; end;
  4 : begin decalx := -decalage; decaly :=  0       ; end;
  5 : begin decalx := -decalage; decaly := -decalage; end;
  6 : begin decalx :=  0       ; decaly := -decalage; end;
  7 : begin decalx := -decalage; decaly := -decalage; end;
  end;
  screen.cursor := crHourglass;
  R0 := GetRValue( ColorToRGB(panel4.color));
  G0 := GetGValue( ColorToRGB(panel4.color));
  B0 := GetBValue( ColorToRGB(panel4.color));
  bmp3.canvas.draw(0,0,bmp2);      // on sauve bmp2 pour dfaire
  For y := 0 to bmp3.height-1 do   // -1 pixel pour dimension y
  for x := 0 to bmp3.width-1 do   // -1 pixel pour dimension x
  begin
    R := Tscana[y, x].RgbtRed;
    G := Tscana[y, x].Rgbtgreen;
    B := Tscana[y, x].Rgbtblue;
    x2 := x+decalx;
    y2 := y+decaly;
    if (y2 >= 1) and (y2 <= Bmp3.height-1) and  // vrif limites
       (x2 >= 1) and (x2 <= Bmp3.width-1) then
    begin
      R2 := Tscana[y2, x2].RgbtRed;
      G2 := Tscana[y2, x2].Rgbtgreen;
      B2 := Tscana[y2, x2].Rgbtblue;
    end
    else
    begin
      R2 := R; // si en dehors on prend le pixel trait
      G2 := G;
      B2 := B;
    end;
    R := R + R0 div 2 - R2;
    G := G + G0 div 2 - G2;
    B := B + B0 div 2 - B2;
    if R > 255 then R := 255 else if R < 0 then r := 0;
    if G > 255 then G := 255 else if G < 0 then g := 0;
    if B > 255 then B := 255 else if B < 0 then b := 0;
    Tscanb[y,x].rgbtred   := R;
    Tscanb[y,x].rgbtgreen := G;
    Tscanb[y,x].rgbtblue  := B;
  end;
  screen.cursor := crDefault;
  affichebmp2;
end;

// Mlange - moyenne des couleurs de Bmp1 et Bmp2 pour donner bmp2.
procedure TForm1.BtmelangeClick(Sender: TObject);
var
  x, y : integer;
  Rowa : Prgbarray;
  Rowb : Prgbarray;
  r, g, b : integer;
  pct : integer;
begin
  bmp3.canvas.draw(0,0,Bmp2); // mmorisation
  pct := updownMel.position;
  For y := 0 to bmp2.height-1 do
  begin
    rowa := Bmp1.scanline[y];
    rowb := Bmp2.scanline[y];
    for x := 0 to bmp2.width-1 do
    begin
      R := rowb[x].Rgbtred   * (100-pct) + rowa[x].RgbtRed   * pct;
      G := rowb[x].Rgbtgreen * (100-pct) + rowa[x].Rgbtgreen * pct;
      B := rowb[x].Rgbtblue  * (100-pct) + rowa[x].Rgbtblue  * pct;
      rowb[x].Rgbtred   := R div 100;
      rowb[x].Rgbtgreen := G div 100;
      rowb[x].Rgbtblue  := B div 100;
    end;
  end;
  affichebmp2;
end;

//-------------  section Updowns
procedure TForm1.UpDownRClick(Sender: TObject; Button: TUDBtnType);
var
  sens : integer;
begin
  If button = btnext then sens := 1 else sens := -1;
  IF chkensemble.checked then
  begin
    UpdownG.position := UpdownG.position+sens;
    UpdownB.position := UpdownB.position+sens;
  end;
end;

procedure TForm1.UpDownGClick(Sender: TObject; Button: TUDBtnType);
var
  sens : integer;
begin
  If button = btnext then sens := 1 else sens := -1;
  IF chkensemble.checked then
  begin
    UpdownR.position := UpdownR.position+sens;
    UpdownB.position := UpdownB.position+sens;
  end;
end;

procedure TForm1.UpDownBClick(Sender: TObject; Button: TUDBtnType);
var
  sens : integer;
begin
  If button = btnext then sens := 1 else sens := -1;
  IF chkensemble.checked then
  begin
    UpdownR.position := UpdownR.position+sens;
    UpdownG.position := UpdownG.position+sens;
  end;
end;

// synchronise ou dsynchronise les 3 updowns
procedure TForm1.ChkensembleClick(Sender: TObject);
var
  valeur : integer;
  choix  : integer;
begin
  IF chkensemble.checked = false then exit;
  valeur := abs(UpdownR.position);
  choix := 1;
  if abs(UpdownG.position) > valeur then
  begin
   valeur := abs(UpdownG.position); choix := 2;
  end;
  if abs(UpdownB.position) > valeur then
  begin
   valeur := abs(UpdownB.position); choix := 3;
  end;
  case choix of
  1 : valeur := UpdownR.position;
  2 : valeur := UpdownG.position;
  3 : valeur := UpdownB.position;
  end;
  UpdownR.position := valeur;
  UpdownG.position := valeur;
  UpdownB.position := valeur;
end;

// remet  zro les 3 updowns
procedure TForm1.BtrazClick(Sender: TObject);
begin
  UpdownR.position := 0;
  UpdownG.position := 0;
  UpdownB.position := 0;
  Chkensemble.checked := false;
end;

procedure TForm1.RadioGroup2Click(Sender: TObject);
begin
  choixfiltre(radiogroup2.itemindex);
end;

Procedure TForm1.choixfiltre(numero : integer);
const
 // Constantes des filtres matrices 3x3 ou 5x5
 // Les deux premiers paramtres sont la taille et le diviseur
F0 : array[0..9] of integer = ( 0, 0, 0,  0, 1, 0,  0, 0, 0,  1); // neutre
F1 : array[0..9] of integer = ( 0, 0, 0,  0, 1, 1,  0, 1, 1,  4); // anti Alias
F2 : array[0..9] of integer = ( 1, 2, 1,  2,20, 2,  1, 2, 1, 32); // Adoucir
F3 : array[0..25] of integer =
    (1,2,4,2,1, 2,4,8,4,2, 4,8,16,8,4, 2,4,6,4,2, 1,2,4,2,1, 98); // Flou Gaus
F4 : array[0..9] of integer = (-1,-2,-1, -2,28,-2, -1,-2,-1, 16); // Durcir
F5 : array[0..9] of integer = ( 0,-1, 0, -1, 5,-1,  0,-1, 0,  1); // Durcir+
F6 : array[0..9] of integer = (-5,-5,-5, -5,42,-5, -5,-5,-5,  2); // Lithographie
F7 : array[0..9] of integer = (-1,-1,-1, -1, 9,-1, -1,-1,-1,  1); // Hi pass
F8 : array[0..9] of integer = ( 0,-1, 0, -1, 1, 1,  0, 1, 0,  1); // relief
F9 : array[0..9] of integer = ( 0, 0, 0,  0, 1, 0,  0, 0, 0,  1); // estamper
FA : array[0..9] of integer = ( 1, 1, 1,  1,-8, 1,  1, 1, 1,  1); // traits
FB : array[0..9] of integer = ( 3, 3, 3,  3,-25,3,  3, 3, 3,  1); // contour
FC : array[0..9] of integer = ( 0,-1,-1,  0, 2, 0,  0, 0, 0,  1); // sculpture
  //charge filtre 3x3
  Procedure F9Load(af : array of integer);
  var
    i, j : integer;
  begin
    for j := 0 to 24 do PF[j] := 0;
    i := 0;
    for j :=  6 to  8 do begin PF[j] := af[i]; inc(i); end;
    for j := 11 to 13 do begin PF[j] := af[i]; inc(i); end;
    for j := 16 to 18 do begin PF[j] := af[i]; inc(i); end;
    DV := af[9];
  end;
  // charge filtre 5x5
  Procedure F25load(af : array of integer);
  var
    i : integer;
  begin
    for i := 0 to 24 do PF[i] := af[i];
    DV := af[25];
  end;

begin
  panelA.visible := false;
  chkgris.checked := false;
  chkneg.checked  := false;
  estamper := false;
  case numero of
  0 : F9Load(F0);
  1 : F9Load(F1);
  2 : F9Load(F2);
  3 : F25load(F3);
  4 : F9Load(F4);
  5 : F9Load(F5);
  6 : F9Load(F6);
  7 : F9Load(F7);
  8 : F9Load(F8);
  9 : begin F9Load(F9); estamper := true; panelA.visible := true; end;
 10 : begin F9Load(FA); chkneg.checked := true; end;
 11 : begin F9Load(FB); chkneg.checked := true; chkgris.checked := true; end;
 12 : begin F9Load(FC); chkneg.checked := true; chkgris.checked := true; end;
 13 : begin F9Load(FC); chkneg.checked := true; chkgris.checked := true;
       vitrage;
      end;
  end;
  Affichefiltre;
end;

Procedure Tform1.affichefiltre;
var
  i : integer;
  s : string;
  c : Tcomponent;
begin
  For i := 0 to 24 do
  begin
    s := inttostr(i);            // Fabrique le nom du composant PF0 .. PF24
    if length(s) < 2 then s := '0'+s;
    s := 'PF'+s;
    c := findcomponent(s);
    if c <> Nil then  (c as Tpanel).caption := inttostr(PF[i]);
  end;
  Pd.caption := inttostr(dv);
end;


procedure TForm1.BtappliquerClick(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  panela.visible := false;
  effets;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  melange := false; paintbox1.cursor := crpipette;
  origine1sur2;
end;

// dplacement d'image  mlanger
procedure TForm1.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  IF melange = false then
  begin
    panel4.color := paintbox1.canvas.pixels[x, y];
  end
  else
  begin
    if form4.radiostyle.itemindex = 2 then exit;   // pavage pas de dplacement
    if (x > posx) and (y > Posy) and (x < posx+posw) and (y < posy+posh) then
    begin
      moving := true;
      startx := x-posx;
      starty := y-posy;
      paintbox1.canvas.pen.mode := pmnotxor;
      paintbox1.canvas.pen.color := cllime;
      paintbox1.canvas.brush.style := bsclear;
      paintbox1.canvas.rectangle(posx, posy, posx+posw, posy+posh);
    end;
  end;
end;

procedure TForm1.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if NOT moving then exit;
  paintbox1.canvas.rectangle(posx, posy, posx+posw, posy+posh);
  posx := x-startx;
  posy := y-starty;
  paintbox1.canvas.rectangle(posx, posy, posx+posw, posy+posh);
end;

procedure TForm1.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  IF not moving then exit;
  paintbox1.canvas.rectangle(posx, posy, posx+posw, posy+posh);
  moving := false;
  Bmp2.canvas.draw(0,0,Bmp3);
  melangebmp(bmp2, bmp0, bounds(posx, posy, posw, posh));
  affichebmp2;
end;

procedure TForm1.Panel4Click(Sender: TObject);
begin
  colordialog1.color := panel4.color;
  If colordialog1.execute then panel4.color := colordialog1.color;
end;

procedure TForm1.PDMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  If y < (PD.height+2) div 2 Then
  begin
    if x < PD.width div 3 then inc(dv,5) else inc(dv);
    if dv = 0 then dv := 1;
  end
  else
  begin
    if x < PD.width div 3 then dec(dv, 5) else  dec(dv);
    if dv = 0 then dv := -1;
  end;
  PD.caption := inttostr(dv);
end;

procedure TForm1.PF00MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i : integer;
  s : string;
begin
  s := (sender as Tpanel).name;
  delete(s,1,2);   // rcupre le numro de panel PFxx
  i := strtoint(s);
  If y < (PF00.height+2) div 2 Then
  begin
    if x < PF00.Width div 3 then inc(PF[i],5) else inc(PF[i]);
  end
  else
  begin
    if x < PF00.width div 3 then dec(PF[i], 5) else  dec(PF[i]);
  end;
  (sender as Tpanel).caption := inttostr(PF[i]);
end;

procedure TForm1.BtgaussClick(Sender: TObject);
var
 GaussR : double;
begin
  melange := false; paintbox1.cursor := crpipette;
  screen.cursor := crHourglass;
  Bmp3.canvas.draw(0,0,bmp2);
  IF updownbruit.position > 12 then updownbruit.position := 12; // 13 maxi
  gaussr := updownbruit.position;
  GBlur(bmp2, GaussR);
  affichebmp2;
  screen.cursor := crDefault;
end;

procedure TForm1.BtnouveauClick(Sender: TObject);
begin
  Form5.caption := 'Nouvelle image';
  IF form5.showmodal = mrok  then
  begin
    bmp0.width  := form5.updown1.position;
    bmp0.height := form5.updown2.position;
    bmp0.pixelformat := pf24bit;
    form5.degrader(bmp0.canvas, rect(0,0, bmp0.width, bmp0.height));
    initbitmaps(0);
    Form1.caption := 'Estampe - Jean-Yves Quinec';
  end;
end;

procedure TForm1.BtsauverClick(Sender: TObject);
begin
  Bmp0.width  := bmp2.width;
  Bmp0.height := bmp2.height;
  Bmp0.canvas.draw(0,0,Bmp2);
  Initbitmaps(0);
end;

procedure TForm1.BtcalculClick(Sender: TObject);
var
  i : integer;
begin
  DV := 0;
  for i := 0 to 24 do DV := DV + PF[i];
  if DV = 0 then DV := 1;
  Pd.caption := inttostr(DV);
end;

procedure TForm1.BtsymOClick(Sender: TObject);
Var
  i : integer;
begin
  for i := 0 to 3 do PF[24-i] := PF[i];
  For i := 5 to 7 do PF[24-i] := PF[i];
  For i := 10 to 11 do PF[24-i] := PF[i];
  PF[24-15] := PF[15];
  affichefiltre;
end;

procedure TForm1.BtsymxClick(Sender: TObject);
Var
  i : integer;
begin
  for i := 0 to 9 do PF[24-i] := PF[i];
  affichefiltre;
end;

procedure TForm1.BtsymYClick(Sender: TObject);
var
 j, i : integer;
begin
  For j := 0 to 4 do  // lignes
  For i := 1 to 2 do  // colonnes
    PF[j*5+2+i] := PF[j*5+2-i];
  affichefiltre;
end;

procedure TForm1.Shap0MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  s : string;
  i :integer;
  c : tcomponent;
  colo : tcolor;
begin
  s := (sender as Tshape).name;
  delete(s,1,4);
  Direction := strtoint(s);
  for i := 0 to 7 do
  begin
    if i = direction then colo := clred else colo := claqua;
    s := 'Shap'+inttostr(i); // Fabrique le nom du composant Shap0 .. Shap7
    c := findcomponent(s);
    if c <> Nil then  (c as Tshape).brush.color := colo;
  end;
  
end;

procedure TForm1.panel5MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Y > (Panel5.height+1) div 2 then dec(decalage) else inc(decalage);
  if decalage < 1 then decalage := 1;
  if decalage > 4 then decalage := 4;
  panel5.caption := inttostr(decalage);
end;

end.
